@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtNetwork module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #ifndef QTLSBACKEND_P_H
 #define QTLSBACKEND_P_H
@@ -67,7 +31,6 @@
 #include <QtNetwork/qssl.h>
 
 #include <QtCore/qloggingcategory.h>
-#include <QtCore/qsharedpointer.h>
 #include <QtCore/qnamespace.h>
 #include <QtCore/qobject.h>
 #include <QtCore/qglobal.h>
@@ -194,8 +157,8 @@ public:
     virtual ~TlsCryptograph();
 
     virtual void init(QSslSocket *q, QSslSocketPrivate *d) = 0;
-    virtual void checkSettingSslContext(QSharedPointer<QSslContext> tlsContext);
-    virtual QSharedPointer<QSslContext> sslContext() const;
+    virtual void checkSettingSslContext(std::shared_ptr<QSslContext> tlsContext);
+    virtual std::shared_ptr<QSslContext> sslContext() const;
 
     virtual QList<QSslError> tlsErrors() const = 0;
 
@@ -363,6 +326,7 @@ public:
     static constexpr const int nameIndexSchannel = 0;
     static constexpr const int nameIndexSecureTransport = 1;
     static constexpr const int nameIndexOpenSSL = 2;
+    static constexpr const int nameIndexCertOnly = 3;
 
     static const QString builtinBackendNames[];
 
@@ -420,13 +384,13 @@ public:
     static void addTustedRoot(QSslSocketPrivate *d, const QSslCertificate &rootCert);// TODO: "addTrusted..."
     // The next one - is a "very important" feature! Kidding ...
     static void setEphemeralKey(QSslSocketPrivate *d, const QSslKey &key);
+
+    virtual void forceAutotestSecurityLevel();
 #endif // QT_CONFIG(ssl)
 
     Q_DISABLE_COPY_MOVE(QTlsBackend)
 };
 
-Q_DECLARE_LOGGING_CATEGORY(lcTlsBackend)
-
 #define QTlsBackend_iid "org.qt-project.Qt.QTlsBackend"
 Q_DECLARE_INTERFACE(QTlsBackend, QTlsBackend_iid);
 
